// Norbert Kern - september 2015 - Making colormaps by use of a color palette collection
 

// render with +kfi1 +kff42 +w1024 +h576 (aspect ratio 16:9) 
     

// Original ColorMapScan macro by Christophe Bouffartique - Insects and... Spiderman  http://www.irtc.org/ftp/pub/stills/2001-06-30/cbspider.zip 

/////////////////////////////////////////////////////////////////////////////////////

#version 3.7; 

#include "stdinc.inc" 

#declare RAD = 4;

global_settings {
        #declare a_g = 2.2;
	assumed_gamma a_g
	subsurface {}
	noise_generator 2
	max_trace_level 4
	#if(RAD > 0)
		radiosity {
			pretrace_start 0.08
			pretrace_end   0.04/RAD
			count 20*RAD
			nearest_count min (20, RAD)
			error_bound 2/RAD
			low_error_factor 0.5
			recursion_limit 1
			gray_threshold 0.4
			minimum_reuse 0.015
			brightness 2
			adc_bailout 0.005
			normal on
			media off
			always_sample off
		}
	#end
}

#declare camloc = <5.28,5.6,9.6>;
#declare lookat = <3.4,2.6,4>;
#declare camang = 20.214*image_width/image_height;

camera { 
        location camloc
        look_at lookat 
        right x*image_width/image_height 
        up y 
        angle camang
}

light_source {
	<-16,20,-32>, rgb 1
        rotate y*-130
}

sky_sphere {
        pigment {
                image_map { 
                        jpeg "environment"              // http://xsi.jankin.com/anisotropic_patterns/environment.jpg
                        map_type 1 
                        interpolate 2
                }
        }
        emission rgb <0.94,0.97,1>
        rotate y*50
}

//________________________________________________________________________________________________________________________________________________________________

//#include "xyzrgb_dragon22k_POV_geom.inc"

#declare set_up =
union {
        cone {<2,-0.01,2>, 0.8, <2,2.3,2>, 0}
        cone {<2,2,-0.01>, 0.8, <2,2,2.3>, 0}
        cone {<-0.01,2,2>, 0.8, <2.3,2,2>, 0} 
        difference {
                box {-0.5, <6.2,3.15,4.3>}
                box {0, <6.201,3.151,4.301>}
        }
        blob {                                          // Friedrich A. Lohmller  "blob{ ... }"  http://www.f-lohmueller.de/pov_tut/all_shapes/shapes620e.htm
                threshold 0.6
                cylinder {<-1,0,0>, <1,0,0>, 1, 1}
                sphere {<0,1.25,0>, 1, 1}
                sphere {<0,0,-1.25>, 1, 1}
                sphere {<0,1.5,0>, 0.5, -1}
                scale 0.7
                rotate <0,45,0> 
                Reorient_Trans (y, 1)
                translate <2,2,2>
        }
        /*object {
                xyzrgb_dragon22k_Material_Node_1_       // http://www.jotero.com/objekte/dragon_jotero.rar
                translate <0,41.3056,0> 
                scale 0.016
                rotate <0,-160,0>
                translate <3.9,0.001*0,0.8>
        }*/ 
}

//________________________________________________________________________________________________________________________________________________________________

#declare nicemaps = 
array [42] {                                             
  89,   98,  218,  379,  432,  564,  632,  728,  730,  741, 
 751,  783,  828,  880,  930, 1080, 1227, 1247, 1343, 1629,     // flip color order in colormap, if array entry is larger than number of colormaps in base image (1250).   
1641, 1663, 1712, 1862, 1901, 1979, 1980, 1982, 2030, 2033,                               
2051, 2074, 2116, 2180, 2291, 2311, 2328, 2333, 2360, 2392,                      
2437, 2442
};

//________________________________________________________________________________________________________________________________________________________________

#declare mapnum = nicemaps [frame_number-1];    // to test this subset of 42 colormaps
//#declare mapnum = frame_number;               // to test all possible colormaps (+kfi1 +kff2500) 

// map     - the image to get colors from
// mapnum  - the number of the color stripe [1 .. 2500] (numbers higher than 1250 automatically flip the order of the colormap entries) 
// colnum  - how many colormap entries [1 .. 256]
// mapmax  - number of color stripes in map (1250)
// reverse - flip the order of the colormap entries by hand [0, 1]

#macro ColorMapScan (map, mapnum, colnum, mapmax, reverse)    
 
#declare PigScan = 
pigment {
        image_map {png map} 
        warp {repeat y flip y}
}

#if (mapnum > mapmax)
        #declare reverse = 1;       
#end
 
#local I = 0;
#while (I < 1)
        #if (reverse)
                [I      rgb eval_pigment (PigScan, <mapnum/mapmax-1/(mapmax*2),1-I-colnum*0.5,0.5>)]
        #else
                [I      rgb eval_pigment (PigScan, <mapnum/mapmax-1/(mapmax*2),I-colnum*0.5,0.5>)]
        #end
        #local I = I+(1/colnum);
#end
 
#end
 
#declare f2p17 =
pigment {

        mandel 32767                     // fractal source by Pavel Tinovsk - http://www.root.cz/clanky/textury-vytvorene-z-fraktalu-a-vyrazu/ 
        frequency 128.501960784314       //                                   - http://i.iinfo.cz/urs-att/fractals-121544346512002.zip
        phase 0
        color_map {ColorMapScan ("2500_colormaps", mapnum, 256, 1250, 0)}
        translate <1.401401443496,-2.70194681500002e-09,0>
        scale <393205.410500566,393205.410506449,1>
}

#declare T1 =
material {
	texture {
                pigment {f2p17}
                normal {
                        pigment_pattern {f2p17}
                        -3
                        accuracy 0.001
                }
	        finish {
	        	reflection {0.1, 0.3}
                        brilliance 2
	        	specular 1
	        	metallic
                        diffuse 0.5
	        }
        }
	interior {ior 1.5}
	scale 0.62
        Reorient_Trans (z, 1)
        translate <2,2,2>
}

object {set_up material {T1} hollow double_illuminate}
 
//________________________________________________________________________________________________________________________________________________________________

// A box showing the used colormap
union {
        box {
                0, <0.2,2.55,0.01>
                texture {
                        pigment {
                                gradient y
                                color_map {ColorMapScan ("2500_colormaps", mapnum, 256, 1250, 0)}
                                warp {repeat y flip y}
                                scale 2.56
                                translate y*-0.01
                        }
                        finish {emission 1 ambient 0 diffuse 0}
                }
        }
        box {
                <-0.025,-0.025,-0.025>, <0.225,2.575,0>
                texture {
                        pigment {color rgb 1}
                        finish {diffuse 1 ambient 0}
                }
        }
        rotate y*12 
        rotate z*7.8
        Reorient_Trans (z, camloc-<5,1.78,1.8>)
        translate <5,0.5,1.8>
        no_shadow
        no_reflection
}

                                                                                                                        





